(ns wanijo.visualisation.routes
  (:require [compojure.core :refer [defroutes wrap-routes
                                    GET POST DELETE]]
            [ring.util.response :as resp]
            [wanijo.framework.routing :refer [register! path]]
            [wanijo.framework.view :as view]
            [wanijo.instance.domain :as domain-instance]
            [wanijo.schema.domain :as domain-schema]
            [wanijo.visualisation
             [domain :as vis-domain]
             [viz :as viz]
             [view :as vis-view]]))

(defn all-instances [req]
  (view/layout!
    :request req
    :content
    [[:h1 "All Instances"]
     (viz/all-instances (vis-domain/all-instance-connections!))]))

(defn all-schemas [req]
  {:content-type "text/json"
   :body (domain-schema/accessible-schemas! (-> req :session :uuid))})

(defn instance [uuid]
  {:content-type "text/json"
   :body (vis-domain/instance! uuid)})

(defn search [term]
  {:content-type "text/json"
   :body (map #(merge %
                      {:urls {:instance (path :vis-get-instance %)}})
              (vis-domain/search! term))})

(defroutes routes
  (GET (register! :vis-explore "/vis/explore/:instance-uuid")
       [instance-uuid :as req]
       (vis-view/index (domain-instance/find-by-uuid! instance-uuid)
                       req))
  (GET (register! :vis-all-instances "/vis/all-instances")
       [:as req]
       (all-instances req))
  (GET (register! :vis-all-schemas "/api/vis/schemas")
       [:as req]
       (all-schemas req))
  (GET (register! :vis-get-instance "/api/vis/instance/:uuid")
       [uuid]
       (instance uuid))
  (GET (register! :vis-search-instance "/api/vis/search/:term")
       [term]
       (search term)))
